@@ -30,7 +30,7 @@ import (
 func TestResolveMarkup(t *testing.T) {
 	c := qt.New(t)
 	cfg := config.New()
-	spec, err := NewContentSpec(cfg, loggers.NewErrorLogger(), afero.NewMemMapFs())
+	spec, err := NewContentSpec(cfg, loggers.NewErrorLogger(), afero.NewMemMapFs(), nil)
 	c.Assert(err, qt.IsNil)
 
 	for i, this := range []struct {
@@ -46,7 +46,6 @@ func TestResolveMarkup(t *testing.T) {
 		{"rst", "rst"},
 		{"pandoc", "pandoc"},
 		{"pdc", "pandoc"},
-		{"mmark", "mmark"},
 		{"html", "html"},
 		{"htm", "html"},
 		{"org", "org"},
